@@ -7,8 +7,8 @@ def datamd5(hdulist):
     sum = hashlib.md5()
     for hdu in hdulist:
         if hdu.data is not None:
-            sum.update(hdu.data.data)
-            pad = '\0' * (2880 - hdu.data.nbytes % 2880)
+            sum.update(bytes(hdu.data.data))
+            pad = b'\0' * (2880 - hdu.data.nbytes % 2880)
             sum.update(pad)
     return sum.hexdigest()
 
