@@ -5,14 +5,18 @@ import tempfile
 import threading
 import collections
 import warnings
+import textwrap
+
+try:
+    from astropy.io import fits
+except:
+    import pyfits as fits
 
-import pyfits
 from . import CPL_recipe
-from . import esorex
-from cpl.frames import FrameList, mkabspath, expandframelist
-from cpl.result import Result, RecipeCrash
-from cpl.param import ParameterList
-from cpl.logger import LogServer
+from .frames import FrameList, mkabspath, expandframelist
+from .result import Result, RecipeCrash
+from .param import ParameterList
+from .logger import LogServer
 
 class Recipe(object):
     '''Pluggable Data Reduction Module (PDRM) from a ESO pipeline. 
@@ -40,6 +44,30 @@ class Recipe(object):
     :func:`cpl.esorex.init()` is called.
     '''
 
+    memory_mode = 0
+    '''CPL memory management mode. The valid values are
+
+    0
+      Use the default system functions for memory handling
+
+    1
+      Exit if a memory-allocation fails, provide checking for memory leaks,
+      limited reporting of memory allocation and limited protection on
+      deallocation of invalid pointers.
+
+    2
+      Exit if a memory-allocation fails, provide checking for memory leaks,
+      extended reporting of memory allocation and protection on deallocation
+      of invalid pointers.
+
+    .. note::
+
+      This variable is only effective before the CPL library was
+      initialized. Even :func:`cpl.Recipe.list()` initializes the library.
+      Therefore it is highly recommended to set this as the first action after
+      importing :mod:`cpl`.
+    '''
+
     def __init__(self, name, filename = None, version = None, threaded = False):
         '''Try to load a recipe with the specified name in the directory
         specified by the class attribute :attr:`Recipe.path` or its 
@@ -62,6 +90,7 @@ class Recipe(object):
         :type threaded: :class:`bool`
 
         '''
+        os.putenv('CPL_MEMORY_MODE', str(Recipe.memory_mode));
         self._recipe = None
         self.__name__ = name
         '''Recipe name.'''
@@ -108,6 +137,8 @@ class Recipe(object):
 
         self.threaded = threaded
 
+        self.mtrace = False
+
         self.__doc__ = self._doc()
 
     @property
@@ -206,22 +237,23 @@ class Recipe(object):
            :obj:`None`.
 
         In order to assing a FITS file to a tag, the file name or the
-        :class:`pyfits.HDUList` is assigned to the calibration attribute:
+        :class:`astropy.io.fits.HDUList` is assigned to the calibration
+        attribute:
 
         >>> muse_scibasic.calib.MASTER_BIAS = 'MASTER_BIAS_0.fits'
 
-        Using :class:`pyfits.HDUList` is useful when it needs to be patched
-        before fed into the recipe. 
+        Using :class:`astropy.io.fits.HDUList` is useful when it needs to be
+        patched before fed into the recipe.
 
-        >>> master_bias = pyfits.open('MASTER_BIAS_0.fits')
+        >>> master_bias = astropy.io.fits.open('MASTER_BIAS_0.fits')
         >>> master_bias[0].header['HIERARCH ESO DET CHIP1 OUT1 GAIN'] = 2.5
         >>> muse_scibasic.calib.MASTER_BIAS = master_bias
 
-        Note that :class:`pyfits.HDUList` objects are stored in temporary
-        files before the recipe is called which may produce some
+        Note that :class:`astropy.io.fits.HDUList` objects are stored in
+        temporary files before the recipe is called which may produce some
         overhead. Also, the CPL then assigns the random temporary file names
         to the FITS keywords ``HIERARCH ESO PRO RECm RAWn NAME`` which should
-        be corrected afterwards if needed. 
+        be corrected afterwards if needed.
 
         To assign more than one frame, put them into a list:
 
@@ -232,7 +264,7 @@ class Recipe(object):
         the map are set are removed from the list, and unknown frame tags are
         silently ignored. The key of the map is the tag name; the values are
         either a string, or a list of strings, containing the file name(s) or
-        the :class:`pyfits.HDUList` objects.
+        the :class:`astropy.io.fits.HDUList` objects.
 
         >>> muse_scibasic.calib = { 'MASTER_BIAS':'master_bias_0.fits', 
         ...                'BADPIX_TABLE':[ 'badpix_1.fits', 'badpix_2.fits' ] }
@@ -334,8 +366,8 @@ class Recipe(object):
         '''Call the recipes execution with a certain input frame.
         
         :param raw: Data input frames.
-        :type raw: :class:`pyfits.HDUlist` or :class:`str` or a :class:`list` 
-            of them, or :class:`dict`
+        :type raw: :class:`astropy.io.fits.HDUlist` or :class:`str` or a 
+            :class:`list` of them, or :class:`dict`
         :param tag: Overwrite the :attr:`tag` attribute (optional).
         :type tag: :class:`str`
         :param threaded: overwrite the :attr:`threaded` attribute (optional).
@@ -357,8 +389,8 @@ class Recipe(object):
         :param env: overwrite environment variables for the recipe call 
             (optional). 
         :type env: :class:`dict`
-        :return: The object with the return frames as :class:`pyfits.HDUList` 
-            objects
+        :return: The object with the return frames as 
+            :class:`astropy.io.fits.HDUList` objects
         :rtype: :class:`cpl.Result`
         :raise: :exc:`exceptions.ValueError` If the invocation parameters
                 are incorrect.
@@ -377,10 +409,11 @@ class Recipe(object):
         '''
         tmpfiles = []
         threaded = ndata.get('threaded', self.threaded)
+        mtrace = ndata.get('mtrace', self.mtrace)
         loglevel = ndata.get('loglevel')
         logname = ndata.get('logname', 'cpl.%s' % self.__name__)
         output_dir = ndata.get('output_dir', self.output_dir)
-        output_format = str if output_dir else pyfits.HDUList
+        output_format = str if output_dir else fits.HDUList
         if output_dir is None:
             output_dir = tempfile.mkdtemp(dir = self.temp_dir, 
                                           prefix = self.__name__ + "-") 
@@ -388,14 +421,14 @@ class Recipe(object):
         raw_frames = self._get_raw_frames(*data, **ndata)
         if len(raw_frames) < 1:
             raise ValueError('No raw frames specified.')
-        input_len = -1 if isinstance(raw_frames[0][1], pyfits.HDUList) else \
+        input_len = -1 if isinstance(raw_frames[0][1], fits.HDUList) else \
             len(raw_frames[0][1]) if isinstance(raw_frames[0][1], list) else -1
         calib_frames = self.calib._aslist(ndata.get('calib'))
         framelist = expandframelist(raw_frames + calib_frames)
         runenv = dict(self.env)
         runenv.update(ndata.get('env', dict()))
         logger = None
-        delete = output_format == pyfits.HDUList
+        delete = output_format == fits.HDUList
         try:
             if (not os.access(output_dir, os.F_OK)):
                 os.makedirs(output_dir)
@@ -409,20 +442,20 @@ class Recipe(object):
             raise
         if not threaded:
             return self._exec(output_dir, parlist, framelist, runenv, 
-                              input_len, logger, output_format, delete)
+                         input_len, logger, output_format, delete, mtrace)
         else:
             return  Threaded(
                 self._exec, output_dir, parlist, framelist, runenv, 
-                input_len, logger, output_format, delete)
+                input_len, logger, output_format, delete, mtrace)
 
     def _exec(self, output_dir, parlist, framelist, runenv,
-              input_len, logger, output_format, delete):
+              input_len, logger, output_format, delete, mtrace):
         try:
             return Result(self._recipe.frameConfig(), output_dir,
                           self._recipe.run(output_dir, parlist, framelist,
                                            list(runenv.items()), 
                                            logger.logfile, logger.level,
-                                           self.memory_dump),
+                                           self.memory_dump, mtrace),
                           input_len, logger, output_format)
         finally:
             self._cleanup(output_dir, logger, delete)
@@ -446,7 +479,7 @@ class Recipe(object):
             elif tag not in m:
                 m[tag] = f
             elif isinstance(m[tag], list) \
-                    and not isinstance(m[tag], pyfits.HDUList):
+                    and not isinstance(m[tag], fits.HDUList):
                 m[tag].append(f)
             else:
                 m[tag] = [ m[tag], f ]
@@ -467,26 +500,26 @@ class Recipe(object):
                 shutil.rmtree(output_dir)
 
     def _doc(self):
-        s = '%s\n\n%s\n\n' % (self.description[0], self.description[1])
+        s = '%s\n\n%s\n\n' % (textwrap.fill(self.description[0]),
+                              textwrap.fill(self.description[1]))
         
         if len(self.param) > 0:
-            r = 'Parameters:\n'
-            maxlen = max(len(p.name) for p in self.param)
-            for p in self.param:
-                r += ' %s: %s (default: %s)\n' % (
-                    p.name.rjust(maxlen), p.__doc__, repr(p.default))
-            r += '\n'
+            r = 'Parameters:\n%s\n' % self._param.__doc__
         else:
             r = 'No parameters\n'
         if self._recipe.frameConfig() is not None:
-            c = 'Calibration frames: %s\n\n' % repr([f.tag for f in self.calib])
+            c = textwrap.fill(repr([f.tag for f in self.calib]),
+                              initial_indent = 'Calibration frames: ',
+                              subsequent_indent = ' ' * 21) + '\n\n'
         else:
             c = ''
         if self.tags is not None:
             t = 'Raw and product frames:\n'
             maxlen = max(len(f) for f in self.tags)
             for f in self.tags:
-                t += ' %s --> %s\n' % (f.rjust(maxlen), repr(self.output[f]))
+                t += textwrap.fill(repr(self.output[f]),
+                                   initial_indent = ' %s --> ' % f.rjust(maxlen),
+                                   subsequent_indent = ' ' * (maxlen + 7)) + '\n'
         else:
             t = ''
         return s + r + c + t + '\n\n'
@@ -502,6 +535,7 @@ class Recipe(object):
         Searches for all recipes in in the directory specified by the class
         attribute :attr:`Recipe.path` or its subdirectories. 
         '''
+        os.putenv('CPL_MEMORY_MODE', str(Recipe.memory_mode));
         plugins = collections.defaultdict(list)
         for f in Recipe.get_libs():
             plugin_f = CPL_recipe.list(f)
@@ -512,6 +546,7 @@ class Recipe(object):
 
     @staticmethod
     def get_recipefilename(name, version = None):
+        os.putenv('CPL_MEMORY_MODE', str(Recipe.memory_mode));
         filename = None
         rversion = -1
         for f in Recipe.get_libs():
